@@ -5,6 +5,8 @@ from dataclasses import dataclass
 from enum import IntEnum
 from typing import Any
 
+from awesomeversion import AwesomeVersion
+
 from .exceptions import WLEDError
 
 
@@ -175,7 +177,7 @@ class Segment:
             length=length,
             on=data.get("on", state_on),
             palette=palette,
-            reverse=data.get("reverse", False),
+            reverse=data.get("rev", False),
             segment_id=segment_id,
             selected=data.get("sel", False),
             speed=data.get("sx", 0),
@@ -256,7 +258,46 @@ class Wifi:
 
 
 @dataclass
-class Info:
+class Filesystem:
+    """Object holding Filesystem information from WLED.
+
+    Args:
+        data: The data from the WLED device API.
+
+    Returns:
+        A Filesystem object.
+    """
+
+    total: int
+    used: int
+    free: int
+    percentage: int
+
+    @staticmethod
+    def from_dict(data: dict[str, Any]) -> Filesystem | None:
+        """Return Filesystem object form WLED API response.
+
+        Args:
+            data: The response from the WLED API.
+
+        Returns:
+            An Filesystem object.
+        """
+        if "fs" not in data:
+            return None
+        filesystem = data.get("fs", {})
+        total = filesystem.get("t", 1)
+        used = filesystem.get("u", 1)
+        return Filesystem(
+            total=total,
+            used=used,
+            free=(total - used),
+            percentage=round((used / total) * 100),
+        )
+
+
+@dataclass
+class Info:  # pylint: disable=too-many-instance-attributes
     """Object holding information from WLED."""
 
     architecture: str
@@ -264,6 +305,7 @@ class Info:
     brand: str
     build_type: str
     effect_count: int
+    filesystem: Filesystem | None
     free_heap: int
     leds: Leds
     live_ip: str
@@ -276,7 +318,9 @@ class Info:
     udp_port: int
     uptime: int
     version_id: str
-    version: str
+    version: AwesomeVersion | None
+    version_latest_beta: AwesomeVersion | None
+    version_latest_stable: AwesomeVersion | None
     websocket: int | None
     wifi: Wifi | None
 
@@ -290,16 +334,25 @@ class Info:
         Returns:
             A info object.
         """
-        websocket = data.get("ws")
-        if websocket == -1:
+        if (websocket := data.get("ws")) == -1:
             websocket = None
 
+        if version := data.get("ver"):
+            version = AwesomeVersion(version)
+
+        if version_latest_stable := data.get("version_latest_stable"):
+            version_latest_stable = AwesomeVersion(version_latest_stable)
+
+        if version_latest_beta := data.get("version_latest_beta"):
+            version_latest_beta = AwesomeVersion(version_latest_beta)
+
         return Info(
             architecture=data.get("arch", "Unknown"),
             arduino_core_version=data.get("core", "Unknown").replace("_", "."),
             brand=data.get("brand", "WLED"),
             build_type=data.get("btype", "Unknown"),
             effect_count=data.get("fxcount", 0),
+            filesystem=Filesystem.from_dict(data),
             free_heap=data.get("freeheap", 0),
             leds=Leds.from_dict(data),
             live_ip=data.get("lip", "Unknown"),
@@ -312,7 +365,9 @@ class Info:
             udp_port=data.get("udpport", 0),
             uptime=data.get("uptime", 0),
             version_id=data.get("vid", "Unknown"),
-            version=data.get("ver", "Unknown"),
+            version=version,
+            version_latest_beta=version_latest_beta,
+            version_latest_stable=version_latest_stable,
             websocket=websocket,
             wifi=Wifi.from_dict(data),
         )
@@ -605,7 +660,7 @@ class Device:
             # we split those out, so we can handle those correctly.
 
             # Nobody cares about 0.
-            _presets.pop("0")
+            _presets.pop("0", None)
 
             presets = [
                 Preset.from_dict(int(preset_id), preset, self.effects, self.palettes)
